#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <resources.h>
#include <gamestate.h>

#define VERSION       "0.1"
#define FPS           50

static int window_width = 800;
static int window_height = 600;
static int fullscreen = 0;
static SDL_Surface * screen;

void init(void)
{
 printf(" --> Initializing...\n");
 // Inicjujemy biblioteki
 if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER | SDL_INIT_AUDIO) < 0)
 {
  printf(" --> SDL initialization failed\n");
  exit(1);
 }
 if (TTF_Init() < 0)
 {
  printf(" --> SDL_ttf initialization failed\n");
  exit(1);
 }
 // Tworzymy okno aplikacji
 printf(" --> Using mode %dx%d (fullscreen: %s)\n", window_width, window_height, (fullscreen ? "Yes" : "No"));
 screen = SDL_SetVideoMode(window_width, window_height, 0, (fullscreen ? (SDL_ANYFORMAT | SDL_FULLSCREEN) : (SDL_ANYFORMAT)));
 // Ustawiamy tytul okna
 SDL_WM_SetCaption("Squash by milyges", NULL);
 // Ukrywamy kursor
 SDL_ShowCursor(0);
 // Ladujemy managera zasobow
 resources_init();
 // Dodajemy zasoby
 //resources_add("background", "data/background.png", rtImage);
 resources_add("ball", "data/ball.png", rtImage);
 resources_add("background", "data/background.png", rtImage);
 resources_add("player1", "data/player1.png", rtImage);
 resources_add("player2", "data/player2.png", rtImage);
 // Inicjujemy manager sceny
 scene_init(screen);
}

void cleanup(void)
{
 // Sprzatamy
 printf(" --> Cleanup...\n");
 SDL_ShowCursor(1);
 scene_cleanup();
 resources_cleanup();
 TTF_Quit();
 SDL_Quit();
}

void parse_args(int argc, char * argv[])
{
 int i;
 for(i=1;i<argc;i++)
 {
  if (!strncmp("--fullscreen=", argv[i], 13))
  {
   if (!strcmp("--fullscreen=yes", argv[i])) fullscreen = 1;
  }
  else if (!strcmp("moo", argv[i]))
  {
   printf(" _____________________________________\n");
   printf("<    Have you played squash today?    >\n");
   printf(" -------------------------------------\n");
   printf("       \\ (__)\n");
   printf("         (oo)\n");
   printf("   /------\\/\n");
   printf("  / |    ||\n");
   printf(" *  /\\---/\\\n");
   printf("    ~~   ~~\n");
   exit(0);
  }
  else
  {
   printf(" --> Unsupported argument %s\n", argv[i]);
   printf(" --> Usage: %s --fullscreen=[yes|no]\n", argv[0]); 
   printf(" -->        --fullscreen=[yes|no] run in window or fullscreen mode\n");
   printf(" -->\n");
   printf(" -->  This squash has Super Cow Powers.\n");
   exit(1);
  }
 }
}

int main(int argc, char * argv[])
{
 SDL_Event event;
 unsigned int start_time; 
 printf("-> Squash by milyges, version %s\n", VERSION);
 parse_args(argc, argv);
 atexit(cleanup);
 init();
 // Przelaczamy sie w stan gry `menu`
 gamestate_select("menu");
 // Glowna petla aplikacji 
 while(1)
 {
  // Tak dlugo jak sa jakies zdarzenia
  if(SDL_PollEvent(&event))
  {
   // Wyjscie z aplikacji obslugujemy w glownej petli
   if(event.type == SDL_QUIT) 
     return 0;
   // Pozostale zdazenia przekazujemy dalej do stanu gry
   if (gamestate_current()->event(&event) != 0) break;
  }
  // Pobieramy czas poczatku renderowania klatki
  start_time = SDL_GetTicks();
  // Wywolujemy procedure przed rysowaniem
  if (gamestate_current()->start_frame() != 0) break; 
  // Rysujemy klatke
  scene_render();
  // Wywolujemy procedure po rysowaniu
  if (gamestate_current()->end_frame() != 0) break;
  // Utrzymujemy stala ilosc klatek na sekunde
  if ((SDL_GetTicks() - start_time) < 1000 / FPS)
  {
   SDL_Delay((1000 / FPS) - (SDL_GetTicks() - start_time));
  }
 }
 gamestate_current()->stop();
 return 0;
}
