#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <resources.h>
#include <gamestate.h>
#include <scene.h>
#include <errno.h>

struct menuitem
{
 struct scene_object obj;
 SDL_Surface * img_normal;
 SDL_Surface * img_high;
 char * caption;
 int (*handler)();
};

static TTF_Font * menufont;
static SDL_Color normal_color = { 0xFF, 0xFF, 0xFF };
static SDL_Color high_color = { 0xFF, 0xFF, 0x00 };
static int selected = 0;
extern int pvp;


static int menuh_pvp(void)
{
 pvp = 1;
 gamestate_select("play");
 return 0;
}

static int menuh_pvc(void)
{
 pvp = 0;
 gamestate_select("play");
 return 0;
}

static int menuh_credits(void)
{
 gamestate_select("credits");
 return 0;
}

static int menuh_quit(void)
{
 return 1;
}
#define MENU_ITEMS     4
static struct menuitem menuitems[MENU_ITEMS] =
{
 { { 0, }, NULL, NULL, "Start game (PvP)", &menuh_pvp },
 { { 0, }, NULL, NULL, "Start game (PvC)", &menuh_pvc },
 { { 0, }, NULL, NULL, "Credits", &menuh_credits },
 { { 0, }, NULL, NULL, "Quit", &menuh_quit }
};

static int menu_start()
{
 int i;
 // Load font
 menufont = TTF_OpenFont("data/menufont.ttf", 48);
 if (!menufont)
 {
  printf(" ---> DEBUG: Can not load menufont\n");
  exit(1);
 }
 
 // TODO: Create background object?
 // Create menu objects
 for(i=0;i<MENU_ITEMS;i++)
 {
  menuitems[i].img_normal = TTF_RenderText_Solid(menufont, menuitems[i].caption, normal_color);
  menuitems[i].img_high = TTF_RenderText_Solid(menufont, menuitems[i].caption, high_color);  
  sprintf(menuitems[i].obj.name, "menuItem%02d", i);
  menuitems[i].obj.w = 0;
  menuitems[i].obj.h = 0;
  if (i != selected) 
     menuitems[i].obj.image = menuitems[i].img_normal;
  else
     menuitems[i].obj.image = menuitems[i].img_high;
  menuitems[i].obj.y = 150 + (i * 50);
  menuitems[i].obj.x = (800 - menuitems[i].obj.image->w) / 2;
  scene_objadd(&menuitems[i].obj);
 }
 
 return 0;
}

static inline void redraw_menu(void)
{
 int i;
 for(i=0;i<MENU_ITEMS;i++)
 {
  if (i != selected) 
     menuitems[i].obj.image = menuitems[i].img_normal;
  else
     menuitems[i].obj.image = menuitems[i].img_high;
 }
}
static int menu_stop()
{
 printf(" ---> DEBUG: menu_stop()\n");
 int i;
 for(i=0;i<MENU_ITEMS;i++)
 {
  scene_objdel(menuitems[i].obj.name);
  SDL_FreeSurface(menuitems[i].img_normal);
  SDL_FreeSurface(menuitems[i].img_high);
 }
 TTF_CloseFont(menufont);
 return 0;
}

static int menu_start_frame()
{
 return 0;
}

int menu_end_frame()
{
 return 0;
}

int menu_event(SDL_Event * event)
{
 if (event->type == SDL_KEYDOWN)
 {
  switch(event->key.keysym.sym)
  {
   case SDLK_UP: if (selected > 0) selected--; break;
   case SDLK_DOWN: if (selected < MENU_ITEMS - 1) selected++; break;
   case SDLK_RETURN: return menuitems[selected].handler();
   case SDLK_ESCAPE: return 1;
  }
  redraw_menu();
 }
 return 0;
}

struct gamestate menustate =
{
 &menu_start,
 &menu_stop,
 &menu_start_frame,
 &menu_end_frame,
 &menu_event,
 "menu"
};
