#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <resources.h>
#include <gamestate.h>
#include <scene.h>
#include <errno.h>
#include <math.h>
#include <time.h>

#define DISTANCE(x1,y1,x2,y2) sqrt(pow( x2 - x1, 2 ) + pow( y2 - y1, 2 ))
#define SWAP_PLAYERS() do { int t = current_p; current_p = last_p; last_p = t; } while(0)
#define UPDATE_SCORE() \
        do { \
            char tmp[64]; \
            sprintf(tmp, "Score:  Green %d:%d Blue", points[0], points[1]); \
            msg_set(0, tmp); \
        } while(0)

static struct step
{
 int x;
 int y;
};

static struct scene_object bg =
{
 0, 0,
 800, 600,
 OBJ_FLAGS_DEFAULT,
 NULL,
 "background"
};

static struct scene_object players[2] =
{
 { 0, 0, 96, 32, OBJ_FLAGS_DEFAULT, NULL, "player1" },
 { 0, 0, 96, 32, OBJ_FLAGS_DEFAULT, NULL, "player2" }
};

static struct scene_object ball =
{
 0, 0, 48, 48, OBJ_FLAGS_DEFAULT, NULL, "ball"
};

static struct scene_object messages[2] = 
{
 { 40, 480, 0, 0, OBJ_FLAGS_DEFAULT, NULL, "score" },
 { 40, 540, 0, 0, OBJ_FLAGS_DEFAULT, NULL, "status" }
};

static struct step ball_step;
static struct step players_step[2];
static int points[2];
static int current_p = 0, last_p = 1;
int pvp;
static int started;
static TTF_Font * font;
static SDL_Color normal_color = { 0xFF, 0xFF, 0xFF };

static inline void msg_set(int id, char * txt)
{
 if (messages[id].image)
    SDL_FreeSurface(messages[id].image);
 messages[id].image = TTF_RenderText_Solid(font, txt, normal_color);
};


static void reset(void)
{
 srand(time(NULL));
 ball_step.y = -(2 + (rand() % 5));
 ball_step.x = 7 + ball_step.y;
 players[0].x = 12;
 players[0].y = 450 - players[1].h;
 players[1].x = 800 - 12 - players[1].w;
 players[1].y = 450 - players[1].h;
 
 if (!last_p)
 {
  ball.x = players[0].x + (players[0].w / 2) - (ball.w / 2);
  ball.y = 450 - players[0].h - ball.h;
 }
 else
 {
  ball.x = players[1].x + (players[0].w / 2) - (ball.w / 2);
  ball.y = 450 - players[1].h - ball.h;
 }
 started = 0;
 msg_set(1, "Press space to start...");
}

static inline int bounced(int p)
{
 if (p == current_p)
 {
  SWAP_PLAYERS();
  return 1;
 }
 points[current_p]++;
 UPDATE_SCORE();
 reset();
 return 0;
}

static inline void check_collision(void)
{
 int i;
 // `boki` piłki
 int leftAv, leftAh, leftB; 
 int rightAv, rightAh, rightB; 
 int topAv, topAh, topB; 
 int bottomAv, bottomAh, bottomB;
 // wspolrzedne paletki
 int x1, y1;
 int x2, y2;
 int x3, y3;
 int x4, y4;
 // Liczymy `boki` piłki
 leftAv = ball.x + (ball.w / 2); 
 rightAv = ball.x + (ball.w / 2); 
 topAv = ball.y; 
 bottomAv = ball.y + ball.h; 
 leftAh = ball.x; 
 rightAh = ball.x + ball.w;
 topAh = ball.y + (ball.h / 2);
 bottomAh = ball.y + (ball.h / 2);
 // liczymy `boki` paletek
 for(i=0;i<2;i++)
 {
  leftB = players[i].x; 
  rightB = players[i].x + players[i].w;
  topB = players[i].y; 
  bottomB = players[i].y + players[i].h; 
  // Liczymy wieszchiolki paletki
  x1 = players[i].x;
  y1 = players[i].y; 
  x2 = players[i].x + players[i].w;
  y2 = players[i].y; 
  x3 = players[i].x;
  y3 = players[i].y + players[i].h;
  x4 = players[i].x + players[i].w;
  y4 = players[i].y + players[i].h;
  if (!((bottomAv <= topB) || (topAv >= bottomB) || (rightAv <= leftB) || (leftAv >= rightB)))
  {
   if (!bounced(i)) return;
   ball_step.y = -ball_step.y;
   ball.y = players[i].y - ball.h - 4;
   ball_step.x += players_step[i].x / 4;
   break;
  }
  else if ((DISTANCE(leftAv, topAh, x1, y1 ) < ball.w / 2) || (DISTANCE(leftAv, topAh, x2, y2) < ball.w / 2) || (DISTANCE(leftAv, topAh, x3, y3 ) < ball.w / 2) || (DISTANCE(leftAv, topAh, x4, y4) < ball.w / 2))
  {
   if (!bounced(i)) return;
   ball_step.y = -ball_step.y;
   ball_step.x = -ball_step.x;
   ball_step.x += players_step[i].x / 4;
   ball.x += ball_step.x;
   ball.y += ball_step.y;
   break;
  }
 }  
}

#define COMP_LVL 6
static void computer_player(void)
{
 if ((ball.y > 450 - COMP_LVL * 45) && (ball_step.y > 0))
 {
  if (current_p == 1)
  {
   players_step[1].x = 0;
   if (ball.x <= players[1].x) players_step[1].x = -8;
   if (ball.x >= players[1].x + players[1].w)  players_step[1].x = 8;
  }
  else
  {
   //if (ball.x <= players[1].x) players_step[1].x = 8 * ((rand() % 2) == 1 ? -1 : 1);
  }
 }
}

static int play_start()
{
 // Load font
 font = TTF_OpenFont("data/font.ttf", 32);
 if (!font)
 {
  printf(" ---> DEBUG: Can not load font\n");
  exit(1);
 }
 bg.image = resources_lookup("background");
 players[0].image = resources_lookup("player1");
 players[1].image = resources_lookup("player2");
 ball.image = resources_lookup("ball");
 UPDATE_SCORE();
 scene_objadd(&bg);
 scene_objadd(&players[0]); 
 scene_objadd(&players[1]);
 scene_objadd(&ball); 
 scene_objadd(&messages[0]);
 scene_objadd(&messages[1]);
 reset();
 return 0;
}

static int play_stop()
{
 return 0;
}

static int play_start_frame()
{
 return 0;
}

int play_end_frame()
{
 if (!started) return 0;
 // Obslugujemy nacisniecie klawiszy
 unsigned char * keystates = SDL_GetKeyState(NULL);
 if (keystates[SDLK_LEFT])
   players_step[0].x = -8;
 else if (keystates[SDLK_RIGHT])
   players_step[0].x = 8;
 else
   players_step[0].x = 0;
 
 if (pvp)
 {
  if (keystates[SDLK_a])
    players_step[1].x = -8;
  else if (keystates[SDLK_d])
    players_step[1].x = 8;
  else
    players_step[1].x = 0;
 }
 else
 {
  computer_player();
 }
 // Wykrywamy kolizje pilka -> sciana
 if (ball.y < 12) ball_step.y = -ball_step.y;
 if (ball.x < 12 || ball.x > 800-12-ball.w) ball_step.x = -ball_step.x;
 // Sprawdzamy czy pilka nie wyleciala poza mape od dolu
 if (ball.y > 450 - ball.h)
 {
  points[last_p]++;
  UPDATE_SCORE();
  reset();
 }
 // Wykrywamy kolizje pilka -> paletka
 // Dlaczego pilka musi byc okrągła....
 check_collision();
 // Przesuwamy pilke
 ball.x += ball_step.x;
 ball.y += ball_step.y;
 //Przesuwamy paletki
 players[0].x += players_step[0].x;
 players[1].x += players_step[1].x;
 // jezeli przesunelismy poza mape poprawiamy
 if (players[0].x < 12) players[0].x = 12;
 if (players[1].x < 12) players[1].x = 12;
 if (players[0].x > 800 - 12 - players[0].w) players[0].x = 800 - 12 - players[0].w;
 if (players[1].x > 800 - 12 - players[1].w) players[1].x = 800 - 12 - players[1].w;
 return 0;
}

int play_event(SDL_Event * event)
{
 if ((event->type == SDL_KEYDOWN) && (event->key.keysym.sym == SDLK_SPACE) && (!started))
 {
  msg_set(1, "");
  started = 1;
 }
 return 0;
}

struct gamestate playstate =
{
 &play_start,
 &play_stop,
 &play_start_frame,
 &play_end_frame,
 &play_event,
 "play"
};
