#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <stdarg.h>
#include <SDL.h>
#include <SDL_image.h>
#include <SDL_ttf.h>
#include <resources.h>
#include <errno.h>

// Manager zasobow

// Maksymalnie 32 obiekty
#define MAX_RESOURCES 32

struct resource * resources[MAX_RESOURCES];

static int do_resources_del(int id)
{
 struct resource * res = resources[id];
 resources[id] = NULL;
 switch(res->type)
 {
  case rtImage: SDL_FreeSurface(res->image); break;
 }
 free(res);
 return 0;
}

int resources_add(char * name, char * path, resource_type_t type)
{
 struct resource * res;
 int err = 0, i;
 res = malloc(sizeof(struct resource));
 if (!res) return -1;
 strcpy(res->name, name);
 res->type = type;
 switch(type)
 {
  case rtImage: // Obiekt jest obrazkiem
  {
   SDL_Surface * img;
   img = IMG_Load(path);
   if (!img)
   { 
    err = ENOENT;
    break;
   }
   printf(" ---> DEBUG: Resource manager: loaded file %s\n", path);
   res->image = SDL_DisplayFormat(img);
   SDL_SetColorKey(res->image, SDL_SRCCOLORKEY, SDL_MapRGB(res->image->format, 0, 0xFF, 0xFF));
   SDL_FreeSurface(img);
   break;
  }
 }
 if (err != 0)
 {
  free(res); 
  return err;
 }
 for(i=0;i<MAX_RESOURCES;i++)
 {
  if (!resources[i])
  {
   resources[i] = res;
   return 0;
  }
 }
 return ENOMEM;
}

void * resources_lookup(char * name)
{
 int i;
 for(i=0;i<MAX_RESOURCES;i++)
 {
  if (!resources[i]) continue;
  if (!strcmp(resources[i]->name, name))
  {
   switch(resources[i]->type)
   {
    case rtImage: return resources[i]->image;
   }
  }
 }
 return NULL;
}

int resources_del(char * name)
{
 int i;
 for(i=0;i<MAX_RESOURCES;i++)
 {
  if (!resources[i]) continue;
  if (!strcmp(resources[i]->name, name))
     return do_resources_del(i);
 }
 return -1;
}

void resources_cleanup(void)
{
 int i;
 for(i=0;i<MAX_RESOURCES;i++)
 {
  if (!resources[i]) continue;
  do_resources_del(i);
 }
}

void resources_init(void)
{
 
}
